﻿Atbar -- Async manager for javascript in node and browsers
===

atbar is a pair of functions (@_ and @$()) that allow for simple control of execution 
flow in async javascript programs. You might notice where the name 'atbar' comes from. Atbar borrows some concepts from creationix/step such as using `this` for easy function calling.

*Some blatant marketing:*

- Allows use of async functions in a linear imperative fashion.
- Allows for parallel or serial execution with no special operators.
- Support for try/catch, if and while flow control.
- A block of code that uses atbar can be called as an async function with callbacks.
- Simple and intuitive. Just two functions. No compiling required.
- Works in any javascript environment.
- No function signature requirements such as node callback style.
- Does not try to eliminate or hide callbacks.  Just makes them easier to use.
- Behaviour is always visible.  No hidden parallel execution.
- Built-in tracing provides easy debugging. Find out where execution disappears.
- Clean code - object oriented - no evals.

How to install ...
---

No specific support for installing is available yet.  The source file, 
atbar.cof is a coffeescript file and is the only file needed.  
The javascript file `atbar.js` is compiled for your convenience.

A quick example ...
---

The following coffeescript example shows serial and parallel execution. First setTimeout A
executes. A calls back to a function where B and C execute in parallel. 
When both B and C have finished then D executes. 
For those of you who are coffeescript-impaired, the JS version follows. 

    atbar.run ->
    	@_ -> 
    		setTimeout @$(), 1000 # A
    	@_ -> 
    		setTimeout @$(), 2000 # B
    		setTimeout @$(), 1000 # C
    	@_ -> 
    		setTimeout @$(), 1000 # D
    		
    atbar.run(function() {
      this._(function() {
        setTimeout(this.$(), 1000); // A
      });
      this._(function() {
        setTimeout(this.$(), 2000);	// B
        setTimeout(this.$(), 1000);	// C
      });
      this._(function() {
        setTimeout(this.$(), 1000);	// D
      });
    });
    
*How it works:* The atbar.run wrapper function creates an atbar object that has the wrapped function
as an attribute.  It then calls the wrapped function and binds the atbar object to the call as
`this`.

The `@_` function is the same function as atbar.run. It creates another atbar object with a wrapped
function. Its object is linked to the first object such that a tree is formed with the first object
as the root.  All the `@_` wrappers are executed as the first function executes. During this execution
`@_` only creates the objects and stores them in the atbar tree. The wrapped functions are not called.

When the function execution completes, the atbar.run function passes control to the first `@_`.  This `@_` executes its function while also binding the atbar object to it as `this`. 
During the execution please notice the `@$()` function calls. `@$` is a callback generator.
Executing `@$()` creates a callback pointing to the following `@_` and binds the atbar object to the callback function using a closure.

When the setTimeout calls the generated callback, control is passed to the second `@_`, which then
proceeds to execute as the first one did. This next function execution launches two setTimeouts,
each with a `@$()` generated callback.  As these callbacks are generated, the callbacks are registered in the atbar object.  When these callbacks happen the `@_` function compares the callbacks to the registration and does not continue until all registered callbacks have occured. When both B and C have called back, then D proceeds as A did.

I apologize for this long-winded explanation of something simple, but I thought I'd dive in and explain completely how it works.  If it doesn't seem simple at this point please try reading this again.

Tracing ...
---

Because all the execution flow of the callbacks goes through method calls of the atbar objects, it was easy to add tracing. You start tracing by adding the 'trace' argument to the first atbar.run call.  Only atbar functions are traced so the listing is concise. This is the console output, with tracing turned on, from the example above (well, almost the same example).

    Starting atbar trace (Version 0.1) ...
       0.001 0    root(first) -> 0       () { 'A'; setTimeout(this.$(), 1000); }
       1.005 0        0(next) -> 1       () { 'B'; setTimeout(this.$(), 2000); 'C'; setTimeout(this.$(), 1000); }
       2.015 1        1(next) -> 2       Callback ignored, waiting for 1 more
       3.005 0        1(next) -> 2       () { 'D'; setTimeout(this.$(), 1000); }
       4.015 0              2 -> root    Nothing to do, exiting ...
       
*Legend:* The columns, from left to right ...

1. Time since start in seconds.
2. Total number of async functions being waited on.
3. Source and destination labels (or `@_` numbers) with the method-used in parens.
4. Source code of destination or a comment.

Wait, there is more!
---

I haven't had time to properly document all the features yet. Here is a list of implemented features ...

- Each `@_` function can have parameters like (err, response) that receive the results of the async function in the callback.
- If the function has the parameter `_throw` as in `(_throw, response)`, then any error value passed in the `_throw` position is automatically thrown. This allows usage of the node error convention.
- Each @_ and @$() can have arbitrary text labels.
	- @$() can direct the generated callback to any `@_` label (yes, this is a goto).
	- When multiple callbacks are "joined", the labels are keys in a hash that contains the results of all callbacks joined.
	- Any `@_` labeled `catch` will catch atbar throws and real exceptions from above.
- Synchronous atbar calls can be made by generating a callback and using it immediately. E.g. `@$('throw')()`.
- `if` and `while` constructs can be easily inserted using standard javascript.
- The `@$()` generator can use any of several algorithms to find the destination `@_`.
	- 'next' is the default. It just goes to the next `@_` as you see in the example.
	- 'loop' goes back to the same `@_`. This is how `while` is implemented.
	- 'throw' goes to the next `@_` with a label of 'catch'.
	- 'first' returns to the first `@_` in the same function.  This allows loops of multiple `@_s`.
	- 'root' starts the whole atbar.run function over again.
	- 'exit' causes atbar.run to finish immediately upon callback.
- `@_` calls can be nested.  This creates a muti-level tree for simple block-structured coding.
- atbar.run can have a callback parameter after its wrapped function.  This is called with the final result when atbar finishes.  This allows atbar.run to be called in a normal async fashion.

Status of project ...
---

Version 0.2 is an early alpha.  While it is completely coded, parts of it are untested.  Only the few test routines in the tests folder have been run.  I wanted to put this out to the node community  early to get feedback. If there are better ways for me to proceed I'd rather know it now than after putting more work in.  Feel free to be blunt about your opinions.

Async utility wars ...
---

If possible, I would like to stay out of the many async utility wars that have erupted on the node discussion board, even though I have started a few in the past. I don't want to compare atbar to other such utilities. I will happily discuss the pros and cons of atbar and I reserve the right to jump into a discussion about async utilites and spout "You should look into atbar".

license ...
---
atbar usage is controlled by the MIT-LICENSE file included in the repository.


